@@ -1,31 +1,17 @@
+// SPDX-FileCopyrightText: 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+// SPDX-FileCopyrightText: 2009 Paul E. McKenney, IBM Corporation.
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
+
 #ifndef _URCU_DEFER_IMPL_H
 #define _URCU_DEFER_IMPL_H
 
 /*
- * urcu-defer-impl.h
- *
  * Userspace RCU header - memory reclamation.
  *
  * TO BE INCLUDED ONLY FROM URCU LIBRARY CODE. See urcu-defer.h for linking
  * dynamically with the userspace rcu reclamation library.
  *
- * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- * Copyright (c) 2009 Paul E. McKenney, IBM Corporation.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- *
  * IBM's contributions to this file may be relicensed under LGPLv2 or later.
  */
 
@@ -33,7 +19,6 @@
 #include <pthread.h>
 #include <stdio.h>
 #include <signal.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 #include <poll.h>
@@ -43,6 +28,7 @@
 
 #include "urcu/futex.h"
 
+#include <urcu/assert.h>
 #include <urcu/compiler.h>
 #include <urcu/arch.h>
 #include <urcu/uatomic.h>
@@ -110,7 +96,7 @@ struct defer_queue {
 /* Do not #define _LGPL_SOURCE to ensure we can emit the wrapper symbols */
 #include <urcu/defer.h>
 
-void __attribute__((destructor)) rcu_defer_exit(void);
+void __attribute__((__destructor__)) rcu_defer_exit(void);
 
 extern void synchronize_rcu(void);
 
@@ -153,8 +139,8 @@ static void mutex_lock_defer(pthread_mutex_t *mutex)
  */
 static void wake_up_defer(void)
 {
-	if (caa_unlikely(uatomic_read(&defer_thread_futex) == -1)) {
-		uatomic_set(&defer_thread_futex, 0);
+	if (caa_unlikely(uatomic_load(&defer_thread_futex) == -1)) {
+		uatomic_store(&defer_thread_futex, 0);
 		if (futex_noasync(&defer_thread_futex, FUTEX_WAKE, 1,
 				NULL, NULL, 0) < 0)
 			urcu_die(errno);
@@ -168,7 +154,7 @@ static unsigned long rcu_defer_num_callbacks(void)
 
 	mutex_lock_defer(&rcu_defer_mutex);
 	cds_list_for_each_entry(index, &registry_defer, list) {
-		head = CMM_LOAD_SHARED(index->head);
+		head = uatomic_load(&index->head);
 		num_items += head - index->tail;
 	}
 	mutex_unlock(&rcu_defer_mutex);
@@ -184,17 +170,17 @@ static void wait_defer(void)
 	/* Write futex before read queue */
 	/* Write futex before read defer_thread_stop */
 	cmm_smp_mb();
-	if (_CMM_LOAD_SHARED(defer_thread_stop)) {
-		uatomic_set(&defer_thread_futex, 0);
+	if (uatomic_load(&defer_thread_stop)) {
+		uatomic_store(&defer_thread_futex, 0);
 		pthread_exit(0);
 	}
 	if (rcu_defer_num_callbacks()) {
 		cmm_smp_mb();	/* Read queue before write futex */
 		/* Callbacks are queued, don't wait. */
-		uatomic_set(&defer_thread_futex, 0);
+		uatomic_store(&defer_thread_futex, 0);
 	} else {
 		cmm_smp_rmb();	/* Read queue before read futex */
-		while (uatomic_read(&defer_thread_futex) == -1) {
+		while (uatomic_load(&defer_thread_futex) == -1) {
 			if (!futex_noasync(&defer_thread_futex, FUTEX_WAIT, -1, NULL, NULL, 0)) {
 				/*
 				 * Prior queued wakeups queued by unrelated code
@@ -238,21 +224,21 @@ static void rcu_defer_barrier_queue(struct defer_queue *queue,
 
 	for (i = queue->tail; i != head;) {
 		cmm_smp_rmb();       /* read head before q[]. */
-		p = CMM_LOAD_SHARED(queue->q[i++ & DEFER_QUEUE_MASK]);
+		p = uatomic_load(&(queue->q[i++ & DEFER_QUEUE_MASK]));
 		if (caa_unlikely(DQ_IS_FCT_BIT(p))) {
 			DQ_CLEAR_FCT_BIT(p);
 			queue->last_fct_out = p;
-			p = CMM_LOAD_SHARED(queue->q[i++ & DEFER_QUEUE_MASK]);
+			p = uatomic_load(&(queue->q[i++ & DEFER_QUEUE_MASK]));
 		} else if (caa_unlikely(p == DQ_FCT_MARK)) {
-			p = CMM_LOAD_SHARED(queue->q[i++ & DEFER_QUEUE_MASK]);
+			p = uatomic_load(&(queue->q[i++ & DEFER_QUEUE_MASK]));
 			queue->last_fct_out = p;
-			p = CMM_LOAD_SHARED(queue->q[i++ & DEFER_QUEUE_MASK]);
+			p = uatomic_load(&(queue->q[i++ & DEFER_QUEUE_MASK]));
 		}
 		fct = queue->last_fct_out;
 		fct(p);
 	}
 	cmm_smp_mb();	/* push tail after having used q[] */
-	CMM_STORE_SHARED(queue->tail, i);
+	uatomic_store(&queue->tail, i);
 }
 
 static void _rcu_defer_barrier_thread(void)
@@ -297,7 +283,7 @@ void rcu_defer_barrier(void)
 
 	mutex_lock_defer(&rcu_defer_mutex);
 	cds_list_for_each_entry(index, &registry_defer, list) {
-		index->last_head = CMM_LOAD_SHARED(index->head);
+		index->last_head = uatomic_load(&index->head);
 		num_items += index->last_head - index->tail;
 	}
 	if (caa_likely(!num_items)) {
@@ -326,16 +312,16 @@ static void _defer_rcu(void (*fct)(void *p), void *p)
 	 * thread.
 	 */
 	head = URCU_TLS(defer_queue).head;
-	tail = CMM_LOAD_SHARED(URCU_TLS(defer_queue).tail);
+	tail = uatomic_load(&URCU_TLS(defer_queue).tail);
 
 	/*
 	 * If queue is full, or reached threshold. Empty queue ourself.
 	 * Worse-case: must allow 2 supplementary entries for fct pointer.
 	 */
 	if (caa_unlikely(head - tail >= DEFER_QUEUE_SIZE - 2)) {
-		assert(head - tail <= DEFER_QUEUE_SIZE);
+		urcu_posix_assert(head - tail <= DEFER_QUEUE_SIZE);
 		rcu_defer_barrier_thread();
-		assert(head - CMM_LOAD_SHARED(URCU_TLS(defer_queue).tail) == 0);
+		urcu_posix_assert(head - uatomic_load(&URCU_TLS(defer_queue).tail) == 0);
 	}
 
 	/*
@@ -359,20 +345,20 @@ static void _defer_rcu(void (*fct)(void *p), void *p)
 			|| p == DQ_FCT_MARK)) {
 		URCU_TLS(defer_queue).last_fct_in = fct;
 		if (caa_unlikely(DQ_IS_FCT_BIT(fct) || fct == DQ_FCT_MARK)) {
-			_CMM_STORE_SHARED(URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK],
+			uatomic_store(&URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK],
 				      DQ_FCT_MARK);
-			_CMM_STORE_SHARED(URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK],
+			uatomic_store(&URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK],
 				      fct);
 		} else {
 			DQ_SET_FCT_BIT(fct);
-			_CMM_STORE_SHARED(URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK],
+			uatomic_store(&URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK],
 				      fct);
 		}
 	}
-	_CMM_STORE_SHARED(URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK], p);
+	uatomic_store(&URCU_TLS(defer_queue).q[head++ & DEFER_QUEUE_MASK], p);
 	cmm_smp_wmb();	/* Publish new pointer before head */
 			/* Write q[] before head. */
-	CMM_STORE_SHARED(URCU_TLS(defer_queue).head, head);
+	uatomic_store(&URCU_TLS(defer_queue).head, head);
 	cmm_smp_mb();	/* Write queue head before read futex */
 	/*
 	 * Wake-up any waiting defer thread.
@@ -380,7 +366,7 @@ static void _defer_rcu(void (*fct)(void *p), void *p)
 	wake_up_defer();
 }
 
-static void *thr_defer(void *args __attribute__((unused)))
+static void *thr_defer(void *args __attribute__((__unused__)))
 {
 	for (;;) {
 		/*
@@ -409,9 +395,19 @@ void defer_rcu(void (*fct)(void *p), void *p)
 static void start_defer_thread(void)
 {
 	int ret;
+	sigset_t newmask, oldmask;
+
+	ret = sigfillset(&newmask);
+	urcu_posix_assert(!ret);
+	ret = pthread_sigmask(SIG_BLOCK, &newmask, &oldmask);
+	urcu_posix_assert(!ret);
 
 	ret = pthread_create(&tid_defer, NULL, thr_defer, NULL);
-	assert(!ret);
+	if (ret)
+		urcu_die(ret);
+
+	ret = pthread_sigmask(SIG_SETMASK, &oldmask, NULL);
+	urcu_posix_assert(!ret);
 }
 
 static void stop_defer_thread(void)
@@ -419,25 +415,25 @@ static void stop_defer_thread(void)
 	int ret;
 	void *tret;
 
-	_CMM_STORE_SHARED(defer_thread_stop, 1);
+	uatomic_store(&defer_thread_stop, 1);
 	/* Store defer_thread_stop before testing futex */
 	cmm_smp_mb();
 	wake_up_defer();
 
 	ret = pthread_join(tid_defer, &tret);
-	assert(!ret);
+	urcu_posix_assert(!ret);
 
-	CMM_STORE_SHARED(defer_thread_stop, 0);
+	uatomic_store(&defer_thread_stop, 0);
 	/* defer thread should always exit when futex value is 0 */
-	assert(uatomic_read(&defer_thread_futex) == 0);
+	urcu_posix_assert(uatomic_load(&defer_thread_futex) == 0);
 }
 
 int rcu_defer_register_thread(void)
 {
 	int was_empty;
 
-	assert(URCU_TLS(defer_queue).last_head == 0);
-	assert(URCU_TLS(defer_queue).q == NULL);
+	urcu_posix_assert(URCU_TLS(defer_queue).last_head == 0);
+	urcu_posix_assert(URCU_TLS(defer_queue).q == NULL);
 	URCU_TLS(defer_queue).q = malloc(sizeof(void *) * DEFER_QUEUE_SIZE);
 	if (!URCU_TLS(defer_queue).q)
 		return -ENOMEM;
@@ -474,7 +470,7 @@ void rcu_defer_unregister_thread(void)
 
 void rcu_defer_exit(void)
 {
-	assert(cds_list_empty(&registry_defer));
+	urcu_posix_assert(cds_list_empty(&registry_defer));
 }
 
 #endif /* _URCU_DEFER_IMPL_H */
