@@ -1,25 +1,11 @@
+// SPDX-FileCopyrightText: 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+// SPDX-FileCopyrightText: 2009 Paul E. McKenney, IBM Corporation.
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
+
 /*
- * urcu.c
- *
  * Userspace RCU library
  *
- * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- * Copyright (c) 2009 Paul E. McKenney, IBM Corporation.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- *
  * IBM's contributions to this file may be relicensed under LGPLv2 or later.
  */
 
@@ -38,6 +24,7 @@
 #include <poll.h>
 
 #include <urcu/config.h>
+#include <urcu/annotate.h>
 #include <urcu/assert.h>
 #include <urcu/arch.h>
 #include <urcu/wfcqueue.h>
@@ -96,24 +83,16 @@ static int urcu_memb_has_sys_membarrier_private_expedited;
 int urcu_memb_has_sys_membarrier = 0;
 #endif
 
-void __attribute__((constructor)) rcu_init(void);
+void __attribute__((__constructor__)) rcu_init(void);
 #endif
 
-#ifdef RCU_MB
+#if defined(RCU_MB)
 void rcu_init(void)
 {
 }
 #endif
 
-#ifdef RCU_SIGNAL
-static int init_done;
-
-void __attribute__((constructor)) rcu_init(void);
-
-static DEFINE_URCU_TLS(int, rcu_signal_was_blocked);
-#endif
-
-void __attribute__((destructor)) rcu_exit(void);
+void __attribute__((__destructor__)) rcu_exit(void);
 static void urcu_call_rcu_exit(void);
 
 /*
@@ -159,9 +138,9 @@ static void mutex_lock(pthread_mutex_t *mutex)
 	while ((ret = pthread_mutex_trylock(mutex)) != 0) {
 		if (ret != EBUSY && ret != EINTR)
 			urcu_die(ret);
-		if (CMM_LOAD_SHARED(URCU_TLS(rcu_reader).need_mb)) {
+		if (uatomic_load(&URCU_TLS(rcu_reader).need_mb)) {
 			cmm_smp_mb();
-			_CMM_STORE_SHARED(URCU_TLS(rcu_reader).need_mb, 0);
+			uatomic_store(&URCU_TLS(rcu_reader).need_mb, 0);
 			cmm_smp_mb();
 		}
 		(void) poll(NULL, 0, 10);
@@ -192,62 +171,13 @@ static void smp_mb_master(void)
 }
 #endif
 
-#ifdef RCU_MB
+#if defined(RCU_MB)
 static void smp_mb_master(void)
 {
 	cmm_smp_mb();
 }
 #endif
 
-#ifdef RCU_SIGNAL
-static void force_mb_all_readers(void)
-{
-	struct urcu_reader *index;
-
-	/*
-	 * Ask for each threads to execute a cmm_smp_mb() so we can consider the
-	 * compiler barriers around rcu read lock as real memory barriers.
-	 */
-	if (cds_list_empty(&registry))
-		return;
-	/*
-	 * pthread_kill has a cmm_smp_mb(). But beware, we assume it performs
-	 * a cache flush on architectures with non-coherent cache. Let's play
-	 * safe and don't assume anything : we use cmm_smp_mc() to make sure the
-	 * cache flush is enforced.
-	 */
-	cds_list_for_each_entry(index, &registry, node) {
-		CMM_STORE_SHARED(index->need_mb, 1);
-		pthread_kill(index->tid, SIGRCU);
-	}
-	/*
-	 * Wait for sighandler (and thus mb()) to execute on every thread.
-	 *
-	 * Note that the pthread_kill() will never be executed on systems
-	 * that correctly deliver signals in a timely manner.  However, it
-	 * is not uncommon for kernels to have bugs that can result in
-	 * lost or unduly delayed signals.
-	 *
-	 * If you are seeing the below pthread_kill() executing much at
-	 * all, we suggest testing the underlying kernel and filing the
-	 * relevant bug report.  For Linux kernels, we recommend getting
-	 * the Linux Test Project (LTP).
-	 */
-	cds_list_for_each_entry(index, &registry, node) {
-		while (CMM_LOAD_SHARED(index->need_mb)) {
-			pthread_kill(index->tid, SIGRCU);
-			(void) poll(NULL, 0, 1);
-		}
-	}
-	cmm_smp_mb();	/* read ->need_mb before ending the barrier */
-}
-
-static void smp_mb_master(void)
-{
-	force_mb_all_readers();
-}
-#endif /* #ifdef RCU_SIGNAL */
-
 /*
  * synchronize_rcu() waiting. Single thread.
  * Always called with rcu_registry lock held. Releases this lock and
@@ -256,14 +186,12 @@ static void smp_mb_master(void)
 static void wait_gp(void)
 {
 	/*
-	 * Read reader_gp before read futex. smp_mb_master() needs to
-	 * be called with the rcu registry lock held in RCU_SIGNAL
-	 * flavor.
+	 * Read reader_gp before read futex.
 	 */
 	smp_mb_master();
 	/* Temporarily unlock the registry lock. */
 	mutex_unlock(&rcu_registry_lock);
-	while (uatomic_read(&rcu_gp.futex) == -1) {
+	while (uatomic_load(&rcu_gp.futex) == -1) {
 		if (!futex_async(&rcu_gp.futex, FUTEX_WAIT, -1, NULL, NULL, 0)) {
 			/*
 			 * Prior queued wakeups queued by unrelated code
@@ -300,7 +228,8 @@ end:
  */
 static void wait_for_readers(struct cds_list_head *input_readers,
 			struct cds_list_head *cur_snap_readers,
-			struct cds_list_head *qsreaders)
+			struct cds_list_head *qsreaders,
+			cmm_annotate_t *group)
 {
 	unsigned int wait_loops = 0;
 	struct urcu_reader *index, *tmp;
@@ -323,7 +252,7 @@ static void wait_for_readers(struct cds_list_head *input_readers,
 		}
 
 		cds_list_for_each_entry_safe(index, tmp, input_readers, node) {
-			switch (urcu_common_reader_state(&rcu_gp, &index->ctr)) {
+			switch (urcu_common_reader_state(&rcu_gp, &index->ctr, group)) {
 			case URCU_READER_ACTIVE_CURRENT:
 				if (cur_snap_readers) {
 					cds_list_move(&index->node,
@@ -350,7 +279,7 @@ static void wait_for_readers(struct cds_list_head *input_readers,
 			if (wait_loops >= RCU_QS_ACTIVE_ATTEMPTS) {
 				/* Read reader_gp before write futex */
 				smp_mb_master();
-				uatomic_set(&rcu_gp.futex, 0);
+				uatomic_store(&rcu_gp.futex, 0);
 			}
 			break;
 		} else {
@@ -378,7 +307,7 @@ static void wait_for_readers(struct cds_list_head *input_readers,
 			if (wait_loops >= RCU_QS_ACTIVE_ATTEMPTS) {
 				/* Read reader_gp before write futex */
 				smp_mb_master();
-				uatomic_set(&rcu_gp.futex, 0);
+				uatomic_store(&rcu_gp.futex, 0);
 			}
 			break;
 		} else {
@@ -407,6 +336,8 @@ static void wait_for_readers(struct cds_list_head *input_readers,
 
 void synchronize_rcu(void)
 {
+	cmm_annotate_define(acquire_group);
+	cmm_annotate_define(release_group);
 	CDS_LIST_HEAD(cur_snap_readers);
 	CDS_LIST_HEAD(qsreaders);
 	DEFINE_URCU_WAIT_NODE(wait, URCU_WAIT_WAITING);
@@ -421,10 +352,11 @@ void synchronize_rcu(void)
 	 * queue before their insertion into the wait queue.
 	 */
 	if (urcu_wait_add(&gp_waiters, &wait) != 0) {
-		/* Not first in queue: will be awakened by another thread. */
+		/*
+		 * Not first in queue: will be awakened by another thread.
+		 * Implies a memory barrier after grace period.
+		 */
 		urcu_adaptative_busy_wait(&wait);
-		/* Order following memory accesses after grace period. */
-		cmm_smp_mb();
 		return;
 	}
 	/* We won't need to wake ourself up */
@@ -449,13 +381,14 @@ void synchronize_rcu(void)
 	 */
 	/* Write new ptr before changing the qparity */
 	smp_mb_master();
+	cmm_annotate_group_mb_release(&release_group);
 
 	/*
 	 * Wait for readers to observe original parity or be quiescent.
 	 * wait_for_readers() can release and grab again rcu_registry_lock
 	 * internally.
 	 */
-	wait_for_readers(&registry, &cur_snap_readers, &qsreaders);
+	wait_for_readers(&registry, &cur_snap_readers, &qsreaders, &acquire_group);
 
 	/*
 	 * Must finish waiting for quiescent state for original parity before
@@ -474,7 +407,8 @@ void synchronize_rcu(void)
 	cmm_smp_mb();
 
 	/* Switch parity: 0 -> 1, 1 -> 0 */
-	CMM_STORE_SHARED(rcu_gp.ctr, rcu_gp.ctr ^ URCU_GP_CTR_PHASE);
+	cmm_annotate_group_mem_release(&release_group, &rcu_gp.ctr);
+	uatomic_store(&rcu_gp.ctr, rcu_gp.ctr ^ URCU_GP_CTR_PHASE);
 
 	/*
 	 * Must commit rcu_gp.ctr update to memory before waiting for quiescent
@@ -497,7 +431,7 @@ void synchronize_rcu(void)
 	 * wait_for_readers() can release and grab again rcu_registry_lock
 	 * internally.
 	 */
-	wait_for_readers(&cur_snap_readers, NULL, &qsreaders);
+	wait_for_readers(&cur_snap_readers, NULL, &qsreaders, &acquire_group);
 
 	/*
 	 * Put quiescent reader list back into registry.
@@ -510,6 +444,7 @@ void synchronize_rcu(void)
 	 * iterates on reader threads.
 	 */
 	smp_mb_master();
+	cmm_annotate_group_mb_acquire(&acquire_group);
 out:
 	mutex_unlock(&rcu_registry_lock);
 	mutex_unlock(&rcu_gp_lock);
@@ -541,52 +476,8 @@ int rcu_read_ongoing(void)
 	return _rcu_read_ongoing();
 }
 
-#ifdef RCU_SIGNAL
-/*
- * Make sure the signal used by the urcu-signal flavor is unblocked
- * while the thread is registered.
- */
-static
-void urcu_signal_unblock(void)
-{
-	sigset_t mask, oldmask;
-	int ret;
-
-	ret = sigemptyset(&mask);
-	urcu_posix_assert(!ret);
-	ret = sigaddset(&mask, SIGRCU);
-	urcu_posix_assert(!ret);
-	ret = pthread_sigmask(SIG_UNBLOCK, &mask, &oldmask);
-	urcu_posix_assert(!ret);
-	URCU_TLS(rcu_signal_was_blocked) = sigismember(&oldmask, SIGRCU);
-}
-
-static
-void urcu_signal_restore(void)
-{
-	sigset_t mask;
-	int ret;
-
-	if (!URCU_TLS(rcu_signal_was_blocked))
-		return;
-	ret = sigemptyset(&mask);
-	urcu_posix_assert(!ret);
-	ret = sigaddset(&mask, SIGRCU);
-	urcu_posix_assert(!ret);
-	ret = pthread_sigmask(SIG_BLOCK, &mask, NULL);
-	urcu_posix_assert(!ret);
-}
-#else
-static
-void urcu_signal_unblock(void) { }
-static
-void urcu_signal_restore(void) { }
-#endif
-
 void rcu_register_thread(void)
 {
-	urcu_signal_unblock();
-
 	URCU_TLS(rcu_reader).tid = pthread_self();
 	urcu_posix_assert(URCU_TLS(rcu_reader).need_mb == 0);
 	urcu_posix_assert(!(URCU_TLS(rcu_reader).ctr & URCU_GP_CTR_NEST_MASK));
@@ -606,8 +497,6 @@ void rcu_unregister_thread(void)
 	URCU_TLS(rcu_reader).registered = 0;
 	cds_list_del(&URCU_TLS(rcu_reader).node);
 	mutex_unlock(&rcu_registry_lock);
-
-	urcu_signal_restore();
 }
 
 #ifdef RCU_MEMBARRIER
@@ -658,57 +547,6 @@ void rcu_init(void)
 }
 #endif
 
-#ifdef RCU_SIGNAL
-static void sigrcu_handler(int signo __attribute__((unused)),
-		siginfo_t *siginfo __attribute__((unused)),
-		void *context __attribute__((unused)))
-{
-	/*
-	 * Executing this cmm_smp_mb() is the only purpose of this signal handler.
-	 * It punctually promotes cmm_barrier() into cmm_smp_mb() on every thread it is
-	 * executed on.
-	 */
-	cmm_smp_mb();
-	_CMM_STORE_SHARED(URCU_TLS(rcu_reader).need_mb, 0);
-	cmm_smp_mb();
-}
-
-/*
- * rcu_init constructor. Called when the library is linked, but also when
- * reader threads are calling rcu_register_thread().
- * Should only be called by a single thread at a given time. This is ensured by
- * holing the rcu_registry_lock from rcu_register_thread() or by running
- * at library load time, which should not be executed by multiple
- * threads nor concurrently with rcu_register_thread() anyway.
- */
-void rcu_init(void)
-{
-	struct sigaction act;
-	int ret;
-
-	if (init_done)
-		return;
-	init_done = 1;
-
-	act.sa_sigaction = sigrcu_handler;
-	act.sa_flags = SA_SIGINFO | SA_RESTART;
-	sigemptyset(&act.sa_mask);
-	ret = sigaction(SIGRCU, &act, NULL);
-	if (ret)
-		urcu_die(errno);
-}
-
-/*
- * Don't unregister the SIGRCU signal handler anymore, because
- * call_rcu threads could still be using it shortly before the
- * application exits.
- * Assertion disabled because call_rcu threads are now rcu
- * readers, and left running at exit.
- * urcu_posix_assert(cds_list_empty(&registry));
- */
-
-#endif /* #ifdef RCU_SIGNAL */
-
 void rcu_exit(void)
 {
 	urcu_call_rcu_exit();
